#!/usr/bin/perl

# NAME
#  bbmon.cgi
#
# VERSION
#  v0.2.CM1 - xhtml 1.0 compliant
#
#
# DESCRIPTION
#  Simplified Web status interface for the Mon resource monitoring system.
#  bbmon.cgi was written to offer a BigBrother-like interface to Mon.
#  It only returns a visual state of the monitored systems. As of today it
#  doesn't offer any interaction, use mon.cgi instead:
#   http://ftp.kernel.org/pub/software/admin/mon/contrib/cgi-bin/mon.cgi/
#
# Requires mon and Mon::Client for proper operation. mon.cgi is a plus
# if you wan't to be able to use the links in bbmon.cgi to the more detailed
# information given by mon.cgi.
#
#  Mon can be found at: http://www.kernel.org/software/mon
#  Mon::Client can be found on CPAN: http://www.cpan.org
#  mon.cgi can be found at: ftp://ftp.kernel.org/pub/software/admin/mon/contrib/cgi-bin/mon.cgi
#
# AUTHORS
#  Nicolas Sayer <Nick@loplop.net>
#   created for and under contract of INRIA / DirDRI / VTHD
#   for the VTHD (Vraiment Trs Haut Dbit) project.
#  Based on the Mon program by Jim Trocki <trockij@transmeta.com>.
#   http://www.kernel.org/software/mon
#  Look & Feel completely inspired from the Big Brother System & Network Monitor
#   http://www.bb4.com/
#    ( this is not at all a BB challenger, the only aim was to give a "BB look"
#      to the totally independent Mon system).
#
#  David Sagnol <David.Sagnol@wanadoo.fr>
#   In 0.2: better generated html. Should be html 4.0 compliant.
#
#  Nathan Gibbs
#   In 0.2.CM1: Xhtml output generation. Should be xhtml 1.0 compliant.
#
#  BBMon can be found at: http://www.loplop.net/projects/bbmon
#
# INSTRUCTIONS
#  bbmon.cgi should be surprisingly easy to install, as long as your Mon setup
#  is working. Actually, if you already have mon.cgi working, you should be
#  able to install bbmon.cgi in 9 seconds, depending on your keyboard type
#  (vi is harder to use with a French keyboard ;-)
#
#  Just copy the bbmon.cgi Perl script in your web-servers cgi-bin directory
#  or any directory allowed to run cgi scripts. (Check out Apache at
#  http://www.apache.org if unfamiliar with CGIs).
#  Also copy the bbimgs directory to wherever you want the gif images to
#  be in.
#  Now then, go and edit the first line of this script to set the path to
#  the Perl executable (you do have Perl ?).
#
#  Next, and finally, go and take a look at the "CONFIGURABLE" section
#  below.
#  The "use lib" line should only be needed if the Mon::Client Perl modules
#  is not in the main Perl installation tree.
#  $imgs: URL path to the gif files location
#  $monuser: Mon user that has at least read-only ability
#  $monpass: his password (default should work?) (see Mon documentation).
#  $monhost: host running the Mon daemon
#  $title: Title of the generated web page
#  $reload: How much time should your browser wait before reloading the page ?
#  $mysite: URL of your site (linked on the logo)
#  $mylogo: URL of your company logo / cat picture / favourite beer or whatever
#  $sysadmin: Your local Gods name
#  $sysadmin: His E-Mail
#  $moncgiurl: The URL to mon.cgi if relevant. This way you'll have
#  links from the bbmon page directly to the mon.cgi specific status
#  reports.
#
#  Most of the configuration stuff affects the cosmetic aspect of the
#  page. It should run "out of the box".
#
#
# BUGS
#  lack of features...
#
#
# FUTURE
#  Let me know about wanted_lists/bugs/anoyances...
#
#           cheers, Nick.
#


# CONFIGURABLE STUFF
use lib '/opt/adm/mon/Perl/lib';

$title="BBMon status (bb-like)";
$imgs="/mon/bbmonimgs";
$moncgiurl="http://127.0.0.1/mon/cgi-bin/mon.cgi";
$monhost="localhost";
$monuser="readonly";
$monpass="public";
$reload="60";
$mysite="http://www.loplop.net/projects/bbmon";
$mylogo="http://www.loplop.net/ploufy.gif";
$sysadmin="Nicolas Sayer";
$sysadminmail="Nick\@loplop.net";
# NPG
$face = "Helvetica, Arial";
# NPG End

# BEGINING OF THE LESS CONFIGURABLE STUFF... (Hackable, yes).

$debug=0;

use Mon::Client;

# NPG
BEGIN {
	# Auto-detect if we are running under mod_perl or CGI.
	$USE_MOD_PERL = exists $ENV{'MOD_PERL'}
	? 1 : 0;
	if ($USE_MOD_PERL) {
		# Use the cgi module and compile all methods at
		# the beginning but only once
		use CGI qw (-compile :standard);
	} else {
		# Use the cgi module and compile all methods only
		# when they are invoked via the autoloader.
		# use CGI qw (-debug) ; #DEBUG
		use CGI qw (:standard) ;
	}
	$CGI::POST_MAX=1024 * 100;	# max 100K posts
	$CGI::DISABLE_UPLOADS = 1;	# no uploads
}
$webpage = new CGI;			# Initialize web page CGI object
print $webpage->header();
# NPG End

%flag = (
	"ok" => "green",
	"unknown" => "yellow",
	"error" => "red",
	"unchecked" => "purple",
	"n.a." => "clear",
);

$mon = new Mon::Client (
	host => $monhost,
	port => 2583,
	username => $monuser,
	password => $monpass,
);

print "Content-type: text/html\n\n" if $debug;
$mon->connect;
$globstatus="ok";

# Creates %services, a hash of arrays, the key containing the watch name,
# pointing to an array of services watched in that group.
foreach $i ($mon->list_watch) {
	push @{ $services{ $i->[0] } }, $i->[1] ;
}

# Creates %hosts, a hash of arrays, matching a watch name to an array of
# hostnames or ip_address contained in that group.
foreach $group (keys %services) {
	push @{ $hosts{ $group } } , $mon->list_group($group);
}

# by default, set all status to unknown, in case script is broken ;-)
foreach $group (keys %services) {
	foreach $service ( 0 .. $#{$services{$group}} ) {
		foreach $i (keys %services) {
			print "service=$services{$group}[$service], grp=$i set to na<br/>\n" if $debug;
			$gstate{$services{$group}[$service]}{$i}="n.a.";
		}
		foreach $host ( 0 .. $#{$hosts{$group}} ) {
			print "\n $group - $services{$group}[$service] - $hosts{$group}[$host] <br/>\n" if $debug;
			$state{$group}{$services{$group}[$service]}{$hosts{$group}[$host]}="unknown";
		}
	}
}



# Now get the state of each service for each host in each group.
# If watch/service is OK, set all hosts to OK,
# of not OK, then match hostnames in the summary line. This works fine if the
# monitors return values according to the mon documentation.
%s = $mon->list_opstatus;
foreach $watch (keys %services) {
	print "Inquiring about hostgroup: $watch<br/>\n" if $debug;
	foreach $service (keys %{$s{$watch}}) {
		print "Inquiring about service: $service<br/>\n" if $debug;
		$opstate=$s{$watch}{$service}{"opstatus"};
		print "OPStatus is :$opstate <br/>\n" if $debug;
		if ( $opstate == 1 ) {
			$gstate{$service}{$watch} = "ok";
			print "Global state of serv=$service/watch=$watch is OK<br/>\n" if $debug;
			foreach $h ( 0 .. $#{$hosts{$watch}} ) {
				$state{$watch}{$service}{$hosts{$watch}[$h]}="ok";
				print "wa=$watch se=$service ho=$hosts{$watch}[$h] set to ok<br/>\n" if $debug;
			}
		} elsif ($s{$watch}{$service}{"opstatus"} == 7) {
			print "wa=$watch se=$service ho=$hosts{$watch}[$h] set to unchecked<br/>\n" if $debug;
			$gstate{$service}{$watch} = "unchecked";
			$globstatus="unchecked" if (globstatus=="ok");
			foreach $h ( 0 .. $#{$hosts{$watch}} ) {
				$state{$watch}{$service}{$hosts{$watch}[$h]}="unchecked";
			}
		} else {
			$gstate{$service}{$watch} = "error";
			$founderr=0;
			foreach $h ( 0 .. $#{$hosts{$watch}} ) {
				$summary=$s{$watch}{$service}{"last_summary"};
# La ligne suivante est a commenter sinon ca ne fonctionne pas dans les versions recentes de Mozilla:-(((
#				print "Summary: $summary<br>\n";  ###
				if ( $s{$watch}{$service}{"last_summary"} =~ /$hosts{$watch}[$h]/ ) {
					$globstatus="error";
					$state{$watch}{$service}{$hosts{$watch}[$h]}="error";
					$founderr=1;
					print "wa=$watch se=$service ho=$hosts{$watch}[$h] set to ERROR<br/>\n" if $debug;
				} else {
					$state{$watch}{$service}{$hosts{$watch}[$h]}="ok";
				}
			}
			if ( $founderr == 0 ) {
				foreach $h ( 0 .. $#{$hosts{$watch}} ) {
					$state{$watch}{$service}{$hosts{$watch}[$h]}="unknown";
				}
			}
		}
	}
}


# Disconnect: We got all the info needed to create the page.
$mon->quit;
$mon->disconnect;

$mytime=scalar localtime;
# NPG
print $webpage->start_html(
	-title=>"$title", -BGCOLOR=>"#000000",
	-BACKGROUND=>"$imgs/bkg-$flag{$globstatus}.gif",
	-TEXT=>"#D8D8BF", -LINK=>"#00FFAA", -VLINK=>"#FFFF44",
	-META=>{'generator'=>"bbmon.cgi",
	'Description'=>"Mon BB-like host status interface", },
	-head=>meta({-http_equiv => 'Refresh', -content => "$reload"}),
	-head=>Link({-rel=>'SHORTCUT ICON', -href=>'/favicon.ico'}),
	# Add css support
	# -style=>{ 'src'=>"$base_link/$stylesheet", 'media'=> "all"},
);
# Start Ouptuting HTML ; should be lynx fiendly.
# Headers / color scheme taken directly from BigBrother.
$webpage->print("<TABLE SUMMARY=Navigation WIDTH=100%><TR>
<TD VALIGN=top ALIGN=left><A HREF=http://bb4.com>
<IMG SRC=$imgs/bb.gif ALT=\"Big Brother\" BORDER=0/></A>
<IMG SRC=$imgs/like.gif ALT=like BORDER=0/>
<A HREF=http://www.kernel.org/software/mon>
<IMG SRC=$imgs/mon.gif ALT=Mon BORDER=0/></A><BR/>");
if ($mysite) {
	if ($mylogo) {
		$webpage->print("<a href=$mysite><img src=$mylogo alt=$mysite border=0/></a>");
	}else{
		$webpage->print("<a href=$mysite>Home</a>");
	}
}
$webpage->print("</TD><TD VALIGN=top ALIGN=right>
<FONT FACE=\"$face\" SIZE=+1 COLOR=silver>
<IMG SRC=$imgs/bbupd.gif ALT=\"Last Update\" WIDTH=107 HEIGHT=24/><BR/>
<B>$mytime</B></FONT></TD></TR></TABLE><BR/><center><H1><I>$title</I></H1>
<TABLE SUMMARY=\"Group Block SumUP\" BORDER=0>\n
<TR><TD VALIGN=middle ROWSPAN=2 ><center>
<FONT COLOR=white SIZE=+1>SUMMARY</FONT></center></TD>");
# NPG End

foreach $i ( sort keys %gstate) {
	print "<TD ALIGN=center VALIGN=bottom WIDTH=45><FONT COLOR=teal SIZE=-1><B>$i</B></FONT></TD>";
}
$nserv=scalar( keys(%gstate))+1;
print "</TR>\n <TR><TD COLSPAN=$nserv><HR WIDTH=100%/></TD></TR>\n";
foreach $i ( sort keys %services) {
	print "<TR><TD><FONT SIZE=+1 COLOR=#FFFFCC FACE=\"$face\"><A HREF=#$i>$i</A></FONT></TD>";
	foreach $j ( sort keys %gstate) {
		# NPG
		print "<td align=center>";
		if ( $flag{$gstate{$j}{$i}} ne "clear") {
			print "<A HREF=$moncgiurl?command=svc_details&amp;args=$i,$j><IMG SRC=$imgs/$flag{$gstate{$j}{$i}}.gif ALT=$gstate{$j}{$i} HEIGHT=16 WIDTH=16 BORDER=0/></A>";
		}else{
			print "<IMG SRC=$imgs/$flag{$gstate{$j}{$i}}.gif ALT=$gstate{$j}{$i} HEIGHT=16 WIDTH=16 BORDER=0/>";
		}
		print "</td>";
		# NPG End
	}
	print "</TR>\n";
}
print "</TABLE><HR/>\n";

# NPG
foreach $i ( sort keys %services) {
	print "<TABLE SUMMARY=\"Group Block\" BORDER=0>\n";
	print "<TR><TD VALIGN=middle ROWSPAN=2><center><FONT COLOR=white SIZE=+1><a name=$i></a><A HREF=$moncgiurl?command=query_group&amp;args=$i>$i</A></FONT></center></TD>";
	foreach $j ( 0 .. $#{$services{$i}} ) {
		print "<TD ALIGN=center VALIGN=bottom WIDTH=45><FONT COLOR=teal SIZE=-1><B><A HREF=$moncgiurl?command=svc_details&amp;args=$i,$services{$i}[$j]>$services{$i}[$j]</A></B></FONT></TD>";
	}
	$nserv=$#{$services{$i}}+1;
	print "</TR>\n<TR><TD COLSPAN=$nserv><HR WIDTH=100%/></TD></TR>\n";
		foreach $k ( 0 .. $#{$hosts{$i}} ) {
			print "<TR><TD><FONT SIZE=+1 COLOR=#FFFFCC FACE=\"$face\">$hosts{$i}[$k]</FONT></TD>";
			foreach $l ( 0 .. $#{$services{$i}} ) {
				print "<TD ALIGN=center><A HREF=$moncgiurl?command=svc_details&amp;args=$i,$services{$i}[$l]><IMG SRC=$imgs/$flag{$state{$i}{$services{$i}[$l]}{$hosts{$i}[$k]}}.gif ALT=$state{$i}{$services{$i}[$l]}{$hosts{$i}[$k]} HEIGHT=16 WIDTH=16 BORDER=0/></A></TD>";
			}
			$webpage->print("</TR>\n");
		}
	$webpage->print("</TABLE>\n");
}

$webpage->print("<BR/><BR/><HR WIDTH=80%/>
<TABLE SUMMARY=Footer WIDTH=80% BORDER=0><TR>");
foreach $i (keys %flag) {
	$webpage->print("<TD ALIGN=center WIDTH=13%>
	<FONT FACE=\"$face\" COLOR=silver>
	<IMG SRC=$imgs/$flag{$i}.gif WIDTH=16 HEIGHT=16 BORDER=0 ALIGN=middle
	ALT=$flag{$i}/>$i</FONT></TD>");
}
$webpage->print("</TR></TABLE>");
if ( $sysadmin ) {
	print "Page maintained by: ";
	if ( $sysadminmail ) {
		print "<A HREF=mailto:$sysadminmail>$sysadmin</A>";
	}else{
		print "$sysadmin";
	}
}
$webpage->print("<HR WIDTH=80%/><BR/><BR/><H4><I>
<FONT FACE=\"$face\" COLOR=silver>
<A HREF=http://www.kernel.org/software/mon>Mon</A> was written by 
<A HREF=mailto:trockij\@transmeta.com>Jim Trocki</A> - 
<A HREF=http://bb4.com>BigBrother</A> is Copyright 
<A HREF=mailto:info\@bb4.com>&copy; 1997-2001 BB4 Technologies Inc.</A> - 
<A HREF=http://www.loplop.net/projects/bbmon>bbmon.cgi</A> was written by 
<A HREF=mailto:Nick\@loplop.net>Nicolas Sayer</A>
</FONT></I></H4></center><p>
<a href=http://validator.w3.org/check/referer>
<img border=0 src=http://www.w3.org/Icons/valid-xhtml10
alt=\"Valid XHTML 1.0!\" height=31 width=88/></a></p>");
print $webpage->end_html;
# NPG End
# the end.
